/// Copyright (c) 2014 Moritz Klammler
///
/// Permission is  hereby granted, free  of charge,  to any person  obtaining a
/// copy of this software and  associated documentation files (the "Software"),
/// to deal in  the Software without restriction,  including without limitation
/// the rights  to use, copy,  modify, merge, publish,  distribute, sublicense,
/// and/or  sell copies  of the  Software, and  to permit  persons to  whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this  permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS",  WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING  BUT NOT LIMITED  TO THE WARRANTIES  OF MERCHANTABILITY,
/// FITNESS FOR  A PARTICULAR PURPOSE  AND NONINFRINGEMENT.  IN NO  EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER  IN AN  ACTION OF CONTRACT,  TORT OR  OTHERWISE, ARISING
/// FROM,  OUT OF  OR IN  CONNECTION  WITH THE  SOFTWARE  OR THE  USE OR  OTHER
/// DEALINGS IN THE SOFTWARE.


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef __GNUC__
#  pragma GCC diagnostic push
#  pragma GCC diagnostic ignored "-Wunused-variable"
#  pragma GCC diagnostic ignored "-Wignored-qualifiers"
#endif
#include <tnt/tnt.h>
#ifdef __GNUC__
#  pragma GCC diagnostic pop
#endif

#include "common.hpp"


namespace /* anonymous */
{
  void
  example(const TNT::Matrix<double>& a,
          const TNT::Matrix<double>& b,
          TNT::Matrix<double>& c,
          const double alpha,
          const double beta)
  {
    // NB: The below expression generates at least 4 temporaries!
    c = mult(alpha, mult(a, transpose(b))) + mult(beta, c);
  }
}


namespace blas_benchmarks
{
  class Benchmark : public AbstractBenchmark
  {
  private:

    TNT::Matrix<double> a_ {};
    TNT::Matrix<double> b_ {};
    TNT::Matrix<double> c_ {};

  protected:

    virtual void
    setUp(const std::size_t n,
          const std::size_t k,
          const std::size_t m) final override
    {
      this->a_ = TNT::Matrix<double> {static_cast<int>(n), static_cast<int>(k)};
      this->b_ = TNT::Matrix<double> {static_cast<int>(m), static_cast<int>(k)};
      this->c_ = TNT::Matrix<double> {static_cast<int>(n), static_cast<int>(m)};
      this->fillRandom_(this->a_);
      this->fillRandom_(this->b_);
      this->fillRandom_(this->c_);
    }

    virtual void
    compute(const double alpha, const double beta) final override
    {
      example(this->a_, this->b_, this->c_, alpha, beta);
    }

    virtual void
    tearDown() final override
    {
      this->a_ = TNT::Matrix<double> {};
      this->b_ = TNT::Matrix<double> {};
      this->c_ = TNT::Matrix<double> {};
    }

    virtual void
    selfTest(const std::size_t n,
             const std::size_t k,
             const std::size_t m,
             const test_matrix_type& a,
             const test_matrix_type& b,
             const test_matrix_type& c,
             const double alpha,
             const double beta,
             test_matrix_type& result) final override
    {
      this->setUp(n, k, m);
      for (std::size_t i = 0; i < n; ++i)
        for (std::size_t j = 0; j < k; ++j)
          this->a_[i][j] = a[i][j];
      for (std::size_t i = 0; i < m; ++i)
        for (std::size_t j = 0; j < k; ++j)
          this->b_[i][j] = b[i][j];
      for (std::size_t i = 0; i < n; ++i)
        for (std::size_t j = 0; j < m; ++j)
          this->c_[i][j] = c[i][j];
      this->compute(alpha, beta);
      for (std::size_t i = 0; i < n; ++i)
        for (std::size_t j = 0; j < m; ++j)
          result[i][j] = this->c_[i][j];
      this->tearDown();
    }

  private:

    void
    fillRandom_(TNT::Matrix<double>& m)
    {
      for (std::size_t j = 0; j < static_cast<std::size_t>(m.num_cols()); ++j)
        for (std::size_t i = 0; i < static_cast<std::size_t>(m.num_rows()); ++i)
          m[i][j] = this->random();
    }
  };
}


int
main(int argc, char * * argv)
{
  std::vector<std::string> args {};
  blas_benchmarks::Benchmark bm {};
  for (int i = 1; i < argc; ++i)
    args.push_back(argv[i]);
  return blas_benchmarks::common_main(args, bm);
}
