dnl -*- coding:utf-8; mode:m4; -*-

dnl SYNOPSYS
dnl
dnl   MY_CXX_BOOST_UBLAS
dnl
dnl DESCRIPTION
dnl
dnl   Checks whether the Boost uBLAS linear algebra C++ librarx can be used.
dnl   Sets the variable 'my_have_boost_ublas' to 'yes' or 'no' and calls
dnl   AC_DEFINE for HAVE_BOOST_UBLAS.  The variable 'my_boost_ublas_version' is
dnl   set to the version number of the found uBLAS library or to the empty
dnl   string, if Boost uBLAS is not found.
dnl
dnl
dnl LICENSE
dnl
dnl   Copyright (c) 2014 Moritz Klammler <moritz.klammler@student.kit.edu>
dnl
dnl   Copying and distribution of this file, with or without modification, are
dnl   permitted in any medium without royalty provided the copyright notice and
dnl   this notice are preserved.  This file is offered as-is, without any
dnl   warranty.
dnl
AC_DEFUN([MY_CXX_BOOST_UBLAS],
[
  my_have_boost_ublas=
  my_boost_ublas_version=
  AC_REQUIRE([AC_PROG_GREP])
  AC_REQUIRE([AC_PROG_SED])
  AC_MSG_CHECKING([for Boost uBLAS])
  AC_LANG_PUSH([C++])
  AC_LINK_IFELSE(
    [AC_LANG_SOURCE([[
#include <boost/numeric/ublas/matrix.hpp>

int main()
{
  typedef size_t index_type;
  typedef double value_type;
  typedef boost::numeric::ublas::matrix<value_type> matrix_type;
  const index_type n = 5;
  const index_type m = 4;
  matrix_type A(n, m);
  matrix_type B(m, n);
  for (index_type i = 0; i < n; ++i)
  {
    for (index_type j = 0; j < m; ++j)
    {
      A(i, j) = static_cast<value_type>(10.0 * i + 0.1 * j);
      B(j, i) = static_cast<value_type>(10.0 * j + 0.1 * i);
    }
  }
  boost::numeric::ublas::prod(A, B);
  return 0;
}
    ]])],
    [
      AC_PREPROC_IFELSE([AC_LANG_SOURCE([[
#include <boost/version.hpp>
static const long boost_ublas_version = (BOOST_VERSION);
        ]])],
        [
          my_boost_ublas_version="$(                                          \
            $GREP 'static const long boost_ublas_version' conftest.i          \
              | $SED -e 's/.*[(]//g' -e 's/[)].*//g' )"
        ],
        [AC_MSG_FAILURE([unexpected preprocessor failure])])
      my_have_boost_ublas=yes
      my_boost_ublas_version=$(printf '%d.%d.%d'                              \
        $(($my_boost_ublas_version / 100000))                                 \
        $((($my_boost_ublas_version / 100) % 1000))                           \
        $(($my_boost_ublas_version % 100)))
      AC_MSG_RESULT([${my_boost_ublas_version}])
      AC_DEFINE([HAVE_BOOST_UBLAS],
                [1],
                [Define if you have Boost uBLAS])
    ],
    [
      my_have_boost_ublas=no
      AC_MSG_RESULT([no])
    ])
  AC_LANG_POP([C++])
])
