dnl -*- coding:utf-8; mode:m4; -*-

dnl SYNOPSYS
dnl
dnl   MY_CXX_TNT
dnl
dnl DESCRIPTION
dnl
dnl   Checks whether the Template Numerical Toolkit (TNT) can be used.  Sets
dnl   the variable 'my_have_tnt' to 'yes' or 'no' and calls AC_DEFINE for
dnl   HAVE_TNT.  The variable 'my_tnt_version' is set to the version number
dnl   of the found TNT library or to the empty string, if TNT is not found.
dnl
dnl
dnl LICENSE
dnl
dnl   Copyright (c) 2014 Moritz Klammler <moritz.klammler@student.kit.edu>
dnl
dnl   Copying and distribution of this file, with or without modification, are
dnl   permitted in any medium without royalty provided the copyright notice and
dnl   this notice are preserved.  This file is offered as-is, without any
dnl   warranty.
dnl
AC_DEFUN([MY_CXX_TNT],
[
  my_have_tnt=
  my_tnt_version=
  AC_REQUIRE([AC_PROG_GREP])
  AC_REQUIRE([AC_PROG_SED])
  AC_MSG_CHECKING([for TNT])
  AC_LANG_PUSH([C++])
  AC_LINK_IFELSE(
    [AC_LANG_SOURCE([[
#ifdef __GNUC__
#  pragma GCC diagnostic push
#  pragma GCC diagnostic ignored "-Wignored-qualifiers"
#endif
#include <tnt/tnt.h>
#ifdef __GNUC__
#  pragma GCC diagnostic pop
#endif

int main()
{
  typedef int index_type;
  typedef double value_type;
  typedef TNT::Array2D<value_type> matrix_type;
  const index_type n {5};
  const index_type m {4};
  matrix_type A {n, m};
  matrix_type B {m, n};
  for (index_type i = 0; i < n; ++i)
  {
    for (index_type j = 0; j < m; ++j)
    {
      A[i][j] = static_cast<value_type>(10.0 * i + 0.1 * j);
      B[j][i] = static_cast<value_type>(10.0 * j + 0.1 * i);
    }
  }
  const matrix_type C = A * B;
  return 0;
}
    ]])],
    [
      AC_PREPROC_IFELSE([AC_LANG_SOURCE([[
#include <tnt/tnt_version.h>
static const char tnt_version[] = {TNT_VERSION_STRING};
        ]])],
        [
          my_tnt_version="$(                                                  \
            $GREP 'static const char tnt_version' conftest.i                  \
              | $SED -e 's/.*[{"]//g' -e 's/["}].*//g'                        \
            )"
        ],
        [AC_MSG_FAILURE([unexpected preprocessor failure])])
      my_have_tnt=yes
      AC_MSG_RESULT([$my_tnt_version])
      AC_DEFINE([HAVE_TNT],
                [1],
                [Define if you have TNT])
    ],
    [
      my_have_tnt=no
      AC_MSG_RESULT([no])
    ])
  AC_LANG_POP([C++])
])
