#!/bin/sh -eu
#! -*- coding:utf-8; mode:shell-script; -*-


all_benchmarks='
cblas
cblas_mp
eigen
eigen_mkl
eigen_mkl_mp
eigen_mp
selfmade
selfmade_mp
tnt
ublas
ublas_mkl
ublas_mkl_mp
ublas_mp
'

if [ -r "${CONFIG_SITE}" ]
then
    ld_library_path="$(. ${CONFIG_SITE} && echo "${LD_LIBRARY_PATH}")"
fi

benchmarks=
samp=10
min=100
max=1000
execprefix='.'
dataprefix='.'
for arg in "$@"
do
    case "${arg}" in
        --help)
            cat <<EOF
usage: run-all.sh [OPTIONS] BENCHMARK...

Runs all benchmarks, writing their output into '*.dat' files and creates a
gnuplot script for plotting them all together.

The following options (only '--option=VALUE' format) are supported:

  --help            show this text and exit
  --samples=N       sample N times (default: ${samp})
  --min-size=N      use minimum matrix size N (default: ${min})
  --max-size=N      use maximum matrix size N (default: ${max})
  --prefix=DIR      find benchmark executables in directory DIR (default: '${execprefix}')
  --output-dir=DIR  place data files in directory DIR (default: '${dataprefix}')
EOF
            exit 0
            ;;
        --samples=*)
            samp="${arg#--samples=}"
            ;;
        --min-size=*)
            min="${arg#--min-size=}"
            ;;
        --max-size=*)
            max="${arg#--max-size=}"
            ;;
        --prefix=*)
            execprefix="${arg#--prefix=}"
            ;;
        --output-dir=*)
            dataprefix="${arg#--output-dir=}"
            ;;
        --*)
            echo "error: unknown option: ${arg}" >&2
            exit 2
            ;;
        *)
            if [ -x "${execprefix}/benchmark_${arg}" ]
            then
                benchmarks="${benchmarks} ${arg}"
            else
                echo "error: no such executable: ${arg}" >&2
                exit 2
            fi
    esac
done
if [ "x${benchmarks}" = x ]
then
    benchmarks="${all_benchmarks}"
fi

gplt_script="${dataprefix}/plot-all.gplt"

mkdir -p "${dataprefix}"

cat <<EOF > "${gplt_script}"
#!/usr/bin/gnuplot

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ #
#  THIS IS A GENERATED FILE - DO NOT EDIT                                     #
#  Generated on $(date) by './run-all.sh'.               #
#  Copyright (C) Moritz Klammler 2014                                         #
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ #

set yrange [0:]
set xlabel 'matrix size'
set ylabel 'effective FPS'

EOF

echo -n 'plot' >> "${gplt_script}"
for b in ${benchmarks}
do
    prog="${execprefix}/benchmark_${b}"
    if [ -x "${prog}" ]
    then
        dat_file_base="${b}.dat"
        dat_file="${dataprefix}/${dat_file_base}"
        echo "${prog} ${samp} ${min} ${max}" >&2
        (
            export LD_LIBRARY_PATH="${ld_library_path-}"
            time "${prog}" ${samp} ${min} ${max} > "${dat_file}"
        )
        echo "" >&2
        printf ' \\\n  %-20s using %s title %s,'                              \
            "'${dat_file_base}'"                                              \
            '(($1 * $2 * $3)**(1.0/3.0)):(($1 * $2 * $3) / $4)'               \
            "'${b}'"                                                          \
            >> "${gplt_script}"
    fi
done

cat <<EOF >> "${gplt_script}"

print 'Hit RET to quit...'
pause -1
EOF

chmod +x "${gplt_script}"
